/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileSystemView;

import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.Resizer.ImageResizer;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.Resizer.ResizeQueueListener;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.DirectorySelectionListener;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.ImageLoadingListener;
import uk.ac.bbsrc.babraham.FocalPoint.Preferences.PrefsDialog;

/*
 * This is the bar which sits at the top of the application.  It comprises
 * the location text field, the image loading status message (the little
 * red/green label on the right), and the button to open the preferences
 * dialog.
 */

public class LocationBar extends JPanel implements DirectorySelectionListener, ResizeQueueListener, ImageLoadingListener, Runnable {

	private JTextField locationField;
	private JLabel workingLabel;
	private JButton prefsButton;
	private boolean imagesResizing = false;
	private int imageLoadingCount = 0;
	private static final LocationBar instance = new LocationBar();
	private boolean working = false;
	
	public static LocationBar getInstance() {
		return instance;
	}
	
	private LocationBar () {
		setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.01;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.NONE;
		
		prefsButton = new JButton("Prefs");
		prefsButton.addActionListener(new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
				new PrefsDialog();
			}
		
		});
		
		add(prefsButton,c);
		
		c.gridx++;
		
		add(new JLabel("Location:"),c);
		c.gridx++;
		c.weightx=0.999;
		c.fill = GridBagConstraints.HORIZONTAL;
		locationField = new JTextField();
		locationField.setEditable(false);
		c.insets = new Insets(0,0,0,4);
		add(locationField,c);

		c.gridx++;
		c.weightx = 0.01;
		c.insets = new Insets(0,0,0,0);
		workingLabel = new JLabel(" Working...");
		workingLabel.setOpaque(true);
		workingLabel.setForeground(Color.GREEN);
		workingLabel.setBackground(Color.GREEN);
		workingLabel.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		add(workingLabel,c);
		
		ImageResizer.getInstance().addListener(this);
	}

	public void directorySelected(File directory) {
		if (!FileSystemView.getFileSystemView().getSystemDisplayName(directory).equals(directory.getName())) {
			locationField.setText(FileSystemView.getFileSystemView().getSystemDisplayName(directory));			
		}
		else {
			locationField.setText(directory.getAbsolutePath());
		}
	}

	private void checkWorkingLabel () {
		if (imagesResizing || imageLoadingCount > 0) {
			if (!working) {
				working = true;
				Thread t = new Thread(this);
				t.start();
			}
		}
		else {
			working = false;
		}
	}
	
	public void isIdle() {
		imagesResizing = false;
		checkWorkingLabel();
	}

	public void isResizing() {
		imagesResizing = true;
		checkWorkingLabel();
	}

	public void ImageLoaded() {
		imageLoadingCount--;
		checkWorkingLabel();
	}

	public void ImageLoading() {
		imageLoadingCount++;
		checkWorkingLabel();
	}

	public void run() {

		Color last = Color.WHITE;
		Color next = Color.RED;
		Color thisColour;
		workingLabel.setBackground(Color.RED);
		workingLabel.setForeground(Color.WHITE);
		
		while (working) {
			try {
				Thread.sleep(300);
				workingLabel.setForeground(next);
				thisColour = next;
				next = last;
				last = thisColour;
			}
			catch (InterruptedException e) {}
		}
		
		workingLabel.setBackground(Color.GREEN);
		workingLabel.setForeground(Color.GREEN);
	}
	
}
