/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.filechooser.FileFilter;

import uk.ac.bbsrc.babraham.FocalPoint.External.DefaultLauncher;
import uk.ac.bbsrc.babraham.FocalPoint.External.ImageJLauncher;

public class ThumbnailPopupMenu extends JPopupMenu implements ActionListener {

	private FileDetailsPanel fileDetails;
	
	public ThumbnailPopupMenu (FileDetailsPanel fileDetails, boolean isImage ) {
		this.fileDetails = fileDetails;
		JMenu openWith = new JMenu("Open with...");
		
		JMenuItem openWithImageJ = new JMenuItem("ImageJ");
		openWithImageJ.setActionCommand("imagej");
		openWithImageJ.addActionListener(this);
		openWith.add(openWithImageJ);
		if (! isImage || ! ImageJLauncher.canUseImageJonThisPlatform()) {
			openWithImageJ.setEnabled(false);
		}
		
		JMenuItem openWithDefault = new JMenuItem("Default Program");
		openWithDefault.setActionCommand("default");
		openWithDefault.addActionListener(this);
		openWith.add(openWithDefault);
		if (! DefaultLauncher.canOpenDefaultOnThisPlatform()) {
			openWithDefault.setEnabled(false);
		}

		add(openWith);

		JMenuItem copyFileTo = new JMenuItem("Copy to...");
		copyFileTo.setActionCommand("copy");
		copyFileTo.addActionListener(this);
		add(copyFileTo);
		
		addSeparator();
		
		JMenuItem closePopup = new JMenuItem("Close menu");
		closePopup.setActionCommand("close");
		closePopup.addActionListener(this);
		add(closePopup);
		
	}

	public void actionPerformed (ActionEvent ae) {

//		System.out.println("Got event of "+ae.getActionCommand());

		if (ae.getActionCommand().equals("imagej")) {
			try {
				ImageJLauncher.openImageWithImageJ(fileDetails.file());
			} 
			catch (IOException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(),"Error Launching ImageJ",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (ae.getActionCommand().equals("default")) {
			try {
				DefaultLauncher.openWithDefaultViewer(fileDetails.file());
			} 
			catch (IOException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(),"Error Launching Default Viewer",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (ae.getActionCommand().equals("copy")) {
			// Need to get the extension of the file
			String [] sections = fileDetails.file().getName().split("\\.");
			File saveFile = getSaveFile(fileDetails.file(),new String [] {sections[sections.length-1]}, sections[sections.length-1]+" files");

			if (saveFile != null) {
				// Try to do the copy.
				try {
					InputStream in = new FileInputStream(fileDetails.file());
					OutputStream out = new FileOutputStream(saveFile);

					byte[] buf = new byte[1024];
					int len;
					while ((len = in.read(buf)) > 0) {
						out.write(buf, 0, len);
					}
					in.close();
					out.close();
				}
				catch (IOException ioe) {
					JOptionPane.showMessageDialog(this, ioe.getMessage(),"Failed to copy file",JOptionPane.ERROR_MESSAGE);				
				}
			}
		}
		
		else if (ae.getActionCommand().equals("close")) {
			setVisible(false);
		}
	}

	private File getSaveFile (final File startingFile, final String [] extensions, final String description) {
		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(startingFile);
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {

			public String getDescription() {
				return description;
			}

			public boolean accept(File f) {
				if (f.isDirectory()) {
					return true;
				}
				for (int i=0;i<extensions.length;i++) {
					if (f.getName().toLowerCase().endsWith(extensions[i].toLowerCase())) {
						return true;
					}
				}
				return false;
			}

		});

		int result = chooser.showSaveDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return null;

		File file = chooser.getSelectedFile();
		boolean extOK = false;
		for (int i=0;i<extensions.length;i++) {
			if (file.getName().toLowerCase().endsWith(extensions[i].toLowerCase())) {
				extOK = true;
				break;
			}
		}
		if (! extOK) {
			file = new File(file.getPath()+"."+extensions[0]);
		}

		// Check if we're stepping on anyone's toes...
		if (file.exists()) {
			int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

			if (answer > 0) {
				return null;
			}
		}
		return file;
	}

}
